rm(list = ls()) # clear history

########################################################
## Packages and libraries                              #
########################################################

# Function to Install and Load R Packages
Install_And_Load <- function(Required_Packages)
{
  Remaining_Packages <- Required_Packages[!(Required_Packages %in% installed.packages()[,"Package"])];
  
  if(length(Remaining_Packages)) 
  {
    install.packages(Remaining_Packages);
  }
  for(package_name in Required_Packages)
  {
    library(package_name,character.only=TRUE,quietly=TRUE);
  }
}

# Specify the list of required packages to be installed and load    
Required_Packages=c("readxl", "lme4", "ggplot2",  "plyr", "dplyr", "performance");

# Call the Function
Install_And_Load(Required_Packages);

########################################################
# SET PATH TO THE DATA ON YOUR DEVICE     
########################################################

aFile<-file.choose() # CHOOSE A FILE FROM YOUR FOLDER CONTAINING THE DATA
data.path = dirname(aFile)# SET THE DATA PATH 
setwd(data.path) # SET THE WORKING DIRECTORY   

pop.df <- readRDS(file = "pop.data.rds") # data for population-level analysis 
# here predators aggregated to coarse taxonomic groupings (e.g. zooplankton) are excluded 

pop.df = subset(pop.df, TL >=2.5) # remove lower-level consumers as some basal resources lack biomass information

# ----------------------------------------------------------------------------------- #
# --------------------- Population-level scaling (one data-point per node)  -------------------#
# ----------------------------------------------------------------------------------- #

# summarise pop dataframe
pop.sum <- ddply(pop.df, .(web),summarise,
                n.pop = length(unique(consumer)),
                max.prey = log10(max(resources)),
                min.prey = log10(min(resources)))

pop.sum$biomass.range <- pop.sum$max.prey - pop.sum$min.prey # calculate biomass range within each web


df1 <- merge(pop.df, pop.sum, by ="web") # merge dataframes
df1 <- subset(df1, n.pop >=5) #  only include webs that have five or more consumers
df1 <- subset(df1, biomass.range >1) #  only include webs with > 1-order of magnitude range in resource biomass 

# linear-mixed effect modeling
# test different random effects structures (r1 - r3)
lme.r1 <- lmer(log.B.con ~ log.B.res * ecosystem + (log.B.res  | web), REML=0, data = df1) 
lme.r2 <- lmer(log.B.con ~ log.B.res  * ecosystem + (1|web) + (0 + log.B.res |web), REML=0, data = df1)
lme.r3 <- lmer(log.B.con ~ log.B.res   * ecosystem + (1|web), REML=0, data = df1)

AIC(lme.r1, lme.r2, lme.r3) # AIC scores
# lme.r1  best model and includes random effects on intercept and slopes (with correlation) 

# test different fixed effects structures (f1 - f3)
lme.f1 <- lmer(log.B.con ~ log.B.res * ecosystem + (log.B.res  | web), REML=0, data = df1)
lme.f2 <- lmer(log.B.con ~ log.B.res + ecosystem + (log.B.res  | web), REML=0, data = df1)
lme.f3 <- lmer(log.B.con ~ log.B.res+ (log.B.res  | web), REML=0, data = df1)

anova(lme.f1, lme.f2, lme.f3) # likelihood ratio tests
# lme.r2 best model (excludes interaction term)

# re-fit with REML to get overall estimates of k (power-law) exponent
lme.final <- lmer(log.B.con ~ log.B.res + ecosystem + (log.B.res  | web), REML=1, data = df1)
summary(lme.final)
model_performance(lme.final)
check_model(lme.final)

# export processed data file
# write.csv(df1, 'pop.data.processed.csv', row.names = F)

# ----------------------------------------------------------------------------------- #
# ------------- ecosystem-level scaling (one data-point per web)  ------------------#
# ----------------------------------------------------------------------------------- #

eco.df <- readRDS(file = "eco.data.rds") # ecosystem data (consumers identified to coarse taxonomic groupings included)
eco.df = subset(eco.df, TL >=2.5) # remove lower-level consumers as some basal resources lack biomass data

# summarise eco.df to obtain information for ecosystem-level analysis 
eco.sum <-  ddply(eco.df, .(web),summarise,
                n = length(unique(consumer)),
                max.prey = log10(max(resources)),
                min.prey = log10(min(resources)))

eco.sum$biomass.range <- eco.sum$max.prey - eco.sum$min.prey

df3 <- merge(eco.df, eco.sum, by ="web") # merge dataframes
df3 = subset(df3, n >=5) #  only include webs that have five or more consumers
df3 = subset(df3, biomass.range >1) #  only include webs with > 1-order of magnitude

# calculate total biomass for consumers and resources in a web
df3.tots <- ddply(df3, .(dataset, ecosystem, web),summarise, 
                    cons = sum(consumer),
                    res = sum(resources))

# ANCOVA analysis
lm.1 <- lm(log10(cons) ~ log10(res) * ecosystem, data = df3.tots)
summary(lm.1)
anova(lm.1) # interaction term non-significant

# re-fit to get overall estimates of k (power-law) exponent
lm.final <- lm(log10(cons) ~ log10(res) + ecosystem, data = df3.tots)
summary(lm.final)
anova(lm.final)
model_performance(lme.final)

# export processed data file
# write.csv(df3.tots, 'eco.data.processed.csv', row.names = F)

# ----------------------------------------------------------------------------------- #
# ---------------------------------------- END   -------------------------------------#
# ----------------------------------------------------------------------------------- #

